'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const turboQuery = require('turbo-query');
const vue = require('vue');

const injectionKey = Symbol(
  "vue-unquery-context"
);
async function useAsyncQuery(key, options) {
  const contextOptions = vue.inject(injectionKey);
  const turboQuery$1 = options?.turbo?.query ?? contextOptions?.turbo?.query ?? turboQuery.query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? turboQuery.mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? turboQuery.subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? turboQuery.forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? turboQuery.abort;
  const refetchOnFocus = options?.refetchOnFocus ?? contextOptions?.refetchOnFocus ?? true;
  const refetchOnConnect = options?.refetchOnConnect ?? contextOptions?.refetchOnConnect ?? true;
  const focusInterval = options?.focusInterval ?? contextOptions?.focusInterval ?? 5e3;
  const turboExpiration = options?.turbo?.expiration ?? contextOptions?.turbo?.expiration ?? turboQuery.expiration;
  const clearOnForget = options?.clearOnForget ?? contextOptions?.clearOnForget ?? false;
  const computedKey = vue.computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = vue.ref();
  const error = vue.ref();
  const isRefetching = vue.ref(false);
  vue.watch(error, (e) => {
    if (e !== void 0)
      throw e;
  });
  if (computedKey.value) {
    resource.value = await turboQuery$1(computedKey.value, {
      stale: true,
      ...contextOptions,
      ...options
    });
  }
  async function refetch() {
    if (!computedKey.value)
      return;
    return await turboQuery$1(computedKey.value, {
      stale: false,
      ...contextOptions,
      ...options
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const lastFocus = vue.ref(new Date());
  function onFocus(listener) {
    if (!refetchOnFocus || typeof window === "undefined")
      return () => {
      };
    const rawHandler = () => {
      const last = lastFocus.value;
      const now = new Date();
      if (now.getTime() - last.getTime() > focusInterval) {
        lastFocus.value = new Date();
        listener();
      }
    };
    window.addEventListener("focus", rawHandler);
    return () => window.removeEventListener("focus", rawHandler);
  }
  function onConnect(listener) {
    if (!refetchOnConnect || typeof window === "undefined")
      return () => {
      };
    window.addEventListener("online", listener);
    return () => window.removeEventListener("online", listener);
  }
  function createFocusAvailable(precision) {
    const isAvailable = vue.ref(new Date().getTime() - lastFocus.value.getTime() > focusInterval);
    const availableIn = vue.ref(focusInterval);
    const interval = setInterval(() => {
      const last = lastFocus.value;
      const now = new Date();
      const availability = focusInterval - (now.getTime() - last.getTime());
      if (availability >= 0)
        availableIn.value = availability;
      else if (availability < 0 && availableIn.value > 0)
        availableIn.value = 0;
      isAvailable.value = now.getTime() - last.getTime() > focusInterval;
    }, precision);
    vue.onUnmounted(() => {
      clearInterval(interval);
    });
    return [isAvailable, availableIn];
  }
  function localExpiration() {
    const key2 = computedKey.value;
    if (!key2)
      return void 0;
    return turboExpiration(key2);
  }
  function createStale(precision) {
    const now = new Date();
    const initialKey = computedKey.value;
    let initialIsStale = true;
    let initialStaleIn = 0;
    if (initialKey) {
      const expiresAt = turboQuery.expiration(initialKey);
      if (expiresAt) {
        const expirationIn = expiresAt.getTime() - now.getTime();
        if (expirationIn >= 0)
          initialStaleIn = expirationIn;
        initialIsStale = expiresAt.getTime() < now.getTime();
      }
    }
    const isStale = vue.ref(initialIsStale);
    const staleIn = vue.ref(initialStaleIn);
    const unsubscribe2 = vue.watch(computedKey, (key2, _old, onCleanup) => {
      const interval = setInterval(() => {
        if (!key2)
          return;
        const expiresAt = turboQuery.expiration(key2);
        if (expiresAt) {
          const now2 = new Date();
          const expirationIn = expiresAt.getTime() - now2.getTime();
          if (expirationIn >= 0)
            staleIn.value = expirationIn;
          else if (expirationIn < 0 && staleIn.value > 0)
            staleIn.value = 0;
          isStale.value = expiresAt.getTime() < now2.getTime();
        }
      }, precision);
      onCleanup(() => {
        clearInterval(interval);
      });
    });
    vue.onUnmounted(() => {
      unsubscribe2();
    });
    return [isStale, staleIn];
  }
  const unsubscribe = vue.watch(
    computedKey,
    async (key2, _old, onCleanup) => {
      isRefetching.value = false;
      if (!key2)
        return;
      const unsubscribeMutate = turboSubscribe(
        key2,
        "mutated",
        (item) => {
          resource.value = item;
        }
      );
      const unsubscribeRefetching = turboSubscribe(
        key2,
        "refetching",
        () => {
          isRefetching.value = true;
        }
      );
      const unsubscribeResolved = turboSubscribe(
        key2,
        "resolved",
        (item) => {
          isRefetching.value = false;
          resource.value = item;
        }
      );
      const unsubscribeErrors = turboSubscribe(
        key2,
        "error",
        (e) => {
          isRefetching.value = false;
          error.value = e;
        }
      );
      const unsubscribeForgotten = turboSubscribe(key2, "forgotten", () => {
        if (clearOnForget)
          resource.value = void 0;
      });
      const unsubscribeFocusRefetch = onFocus(() => {
        refetch();
      });
      const unsubscribeConnectRefetch = onConnect(() => {
        refetch();
      });
      resource.value = await turboQuery$1(key2, {
        stale: true,
        ...contextOptions,
        ...options
      });
      onCleanup(() => {
        unsubscribeMutate();
        unsubscribeRefetching();
        unsubscribeResolved();
        unsubscribeErrors();
        unsubscribeForgotten();
        unsubscribeFocusRefetch();
        unsubscribeConnectRefetch();
      });
    },
    { immediate: true }
  );
  if (vue.getCurrentInstance())
    vue.onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: vue.readonly(error),
      isRefetching: vue.readonly(isRefetching),
      lastFocus: vue.readonly(lastFocus),
      createFocusAvailable,
      expiration: localExpiration,
      createStale
    }
  ];
}
function useQuery(key, options) {
  const contextOptions = vue.inject(injectionKey);
  const turboQuery$1 = options?.turbo?.query ?? contextOptions?.turbo?.query ?? turboQuery.query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? turboQuery.mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? turboQuery.subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? turboQuery.forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? turboQuery.abort;
  const refetchOnFocus = options?.refetchOnFocus ?? contextOptions?.refetchOnFocus ?? true;
  const refetchOnConnect = options?.refetchOnConnect ?? contextOptions?.refetchOnConnect ?? true;
  const focusInterval = options?.focusInterval ?? contextOptions?.focusInterval ?? 5e3;
  const turboExpiration = options?.turbo?.expiration ?? contextOptions?.turbo?.expiration ?? turboQuery.expiration;
  const clearOnForget = options?.clearOnForget ?? contextOptions?.clearOnForget ?? false;
  const immediate = options?.immediate ?? contextOptions?.immediate ?? true;
  const computedKey = vue.computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = vue.ref();
  const error = vue.ref();
  const isRefetching = vue.ref(false);
  (async () => {
    if (!computedKey.value || !immediate)
      return;
    resource.value = await turboQuery$1(computedKey.value, {
      stale: true,
      ...contextOptions,
      ...options
    });
  })();
  async function refetch() {
    if (!computedKey.value)
      return;
    return await turboQuery$1(computedKey.value, {
      stale: false,
      ...contextOptions,
      ...options
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const lastFocus = vue.ref(new Date());
  function onFocus(listener) {
    if (!refetchOnFocus || typeof window === "undefined")
      return () => {
      };
    const rawHandler = () => {
      const last = lastFocus.value;
      const now = new Date();
      if (now.getTime() - last.getTime() > focusInterval) {
        lastFocus.value = new Date();
        listener();
      }
    };
    window.addEventListener("focus", rawHandler);
    return () => window.removeEventListener("focus", rawHandler);
  }
  function onConnect(listener) {
    if (!refetchOnConnect || typeof window === "undefined")
      return () => {
      };
    window.addEventListener("online", listener);
    return () => window.removeEventListener("online", listener);
  }
  function createFocusAvailable(precision) {
    const isAvailable = vue.ref(new Date().getTime() - lastFocus.value.getTime() > focusInterval);
    const availableIn = vue.ref(focusInterval);
    const interval = setInterval(() => {
      const last = lastFocus.value;
      const now = new Date();
      const availability = focusInterval - (now.getTime() - last.getTime());
      if (availability >= 0)
        availableIn.value = availability;
      else if (availability < 0 && availableIn.value > 0)
        availableIn.value = 0;
      isAvailable.value = now.getTime() - last.getTime() > focusInterval;
    }, precision);
    vue.onUnmounted(() => {
      clearInterval(interval);
    });
    return [isAvailable, availableIn];
  }
  function localExpiration() {
    const key2 = computedKey.value;
    if (!key2)
      return void 0;
    return turboExpiration(key2);
  }
  function createStale(precision) {
    const now = new Date();
    const initialKey = computedKey.value;
    let initialIsStale = true;
    let initialStaleIn = 0;
    if (initialKey) {
      const expiresAt = turboQuery.expiration(initialKey);
      if (expiresAt) {
        const expirationIn = expiresAt.getTime() - now.getTime();
        if (expirationIn >= 0)
          initialStaleIn = expirationIn;
        initialIsStale = expiresAt.getTime() < now.getTime();
      }
    }
    const isStale = vue.ref(initialIsStale);
    const staleIn = vue.ref(initialStaleIn);
    const unsubscribe2 = vue.watch(computedKey, (key2, _old, onCleanup) => {
      const interval = setInterval(() => {
        if (!key2)
          return;
        const expiresAt = turboQuery.expiration(key2);
        if (expiresAt) {
          const now2 = new Date();
          const expirationIn = expiresAt.getTime() - now2.getTime();
          if (expirationIn >= 0)
            staleIn.value = expirationIn;
          else if (expirationIn < 0 && staleIn.value > 0)
            staleIn.value = 0;
          isStale.value = expiresAt.getTime() < now2.getTime();
        }
      }, precision);
      onCleanup(() => {
        clearInterval(interval);
      });
    });
    vue.onUnmounted(() => {
      unsubscribe2();
    });
    return [isStale, staleIn];
  }
  const unsubscribe = vue.watch(
    computedKey,
    async (key2, _old, onCleanup) => {
      isRefetching.value = false;
      if (!key2)
        return;
      const unsubscribeMutate = turboSubscribe(
        key2,
        "mutated",
        (item) => {
          resource.value = item;
        }
      );
      const unsubscribeRefetching = turboSubscribe(
        key2,
        "refetching",
        () => {
          isRefetching.value = true;
        }
      );
      const unsubscribeResolved = turboSubscribe(
        key2,
        "resolved",
        (item) => {
          isRefetching.value = false;
          resource.value = item;
        }
      );
      const unsubscribeErrors = turboSubscribe(
        key2,
        "error",
        (e) => {
          isRefetching.value = false;
          error.value = e;
        }
      );
      const unsubscribeForgotten = turboSubscribe(key2, "forgotten", () => {
        if (clearOnForget)
          resource.value = void 0;
      });
      const unsubscribeFocusRefetch = onFocus(() => {
        refetch();
      });
      const unsubscribeConnectRefetch = onConnect(() => {
        refetch();
      });
      resource.value = await turboQuery$1(key2, {
        stale: true,
        ...contextOptions,
        ...options
      });
      onCleanup(() => {
        unsubscribeMutate();
        unsubscribeRefetching();
        unsubscribeResolved();
        unsubscribeErrors();
        unsubscribeForgotten();
        unsubscribeFocusRefetch();
        unsubscribeConnectRefetch();
      });
    },
    { immediate }
  );
  if (vue.getCurrentInstance())
    vue.onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: vue.readonly(error),
      isRefetching: vue.readonly(isRefetching),
      lastFocus: vue.readonly(lastFocus),
      createFocusAvailable,
      expiration: localExpiration,
      createStale
    }
  ];
}

exports.injectionKey = injectionKey;
exports.useAsyncQuery = useAsyncQuery;
exports.useQuery = useQuery;
